-- ========================================================================
-- Deployment Functions
-- ========================================================================
function GetSide(playerside)
    local result = playerside
    if playerside ~= nil then
        local retval, side =  xpcall(ScenEdit_GetSideOptions, SeaLaneErrorhandler,{side=playerside}) ;
        if not retval or side == nil then
            ScenEdit_AddSide({name=playerside})
            result = ScenEdit_GetSideOptions({side=playerside})
        end
    end
    return result
end


function PopulateSealanes (ships, sealanes, unitsper100nm, shipclass, shipside)
    -- Adds ships along the sea lanes at a number per 100nm of the ship classes to the side identified.
    -- PopulateSealanes(GLOBAL_COMMERCIALSHIPS, sealanes, 1, {'Container ship', 'Cargo ship', 'Bulk carrier'}, 'neutral')
    local deploymentdefinition = {}
    local sealaneships = {};
    if ships ~= nil and sealanes ~= nil and unitsper100nm ~= nil and shipclass ~= nil and shipside then
        local playerside = GetSide(shipside)
        if playerside ~= nil then
            for k, v in pairs(sealanes) do
                local numships = math.floor((v.length/100) * unitsper100nm)
                for i=1,Count(shipclass) do
                    table.insert(deploymentdefinition, {class=shipclass[i], quantity= numships, sealane = v.course, side=shipside})
                end
            end
            sealaneships = DeployTraffic(ships, deploymentdefinition)
        else
            print('Side ' .. shipside .. ' is not present')
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return sealaneships
end

function SeaLaneErrorhandler( err )
    print( "ERROR:", err )
 end

function DeployTraffic(ships, ...)
    local args = {...}
    args = args[1]
    if args == nil or Count(args) == 0 then
        return
    end
    local sealaneships = {};
    if ships ~= nil then
        for k,v in pairs(args) do
            if v.quantity ~= nil and v.quantity >0 and v.sealane ~= nil
                and v.class ~= nil and v.side ~= nil then
                local qty2deploy = v.quantity
                local shippool = FilterTable(ships, {{class= v.class}})
                local shipside = GetSide(v.side)
                local usednames = {}
                if shippool ~= nil and Count(shippool) > 0 then
                    while (qty2deploy > 0)
                    do
                        local catalogship = shippool[math.random(1,Count(shippool))]
                        local shipname = catalogship.unitname
                        -- de-dupe name
                        local suffixcounter = 1
                        while( InTable(usednames, shipname) ) do
                            shipname = catalogship.unitname ..  "-" .. suffixcounter
                            suffixcounter = suffixcounter + 1
                        end
                        qty2deploy = qty2deploy - 1
                        table.insert(usednames, shipname)
                        -- Plot the ship
                        local startloc, newcourse = CourseFromSeaLane(v.sealane)
                        if startloc ~= nil and newcourse ~= nil then
                            local callsuccess, shipT = xpcall(ScenEdit_AddUnit, SeaLaneErrorhandler,
                             {type ='Ship', name = shipname, dbid = catalogship.dbid, side = shipside, throttle='cruise', Latitude=startloc.latitude,Longitude=startloc.longitude}) ;
                            if callsuccess and shipT ~= nil and shipT.guid ~= nil then
                                ScenEdit_SetUnit({guid=shipT.guid,course=newcourse})
                                AddInitialHeading(shipT.guid)
                                table.insert(sealaneships, shipT)
                            end
                        end
                    end
                else
                    print("The class " .. v.class .. " was not present in the commercialship list")
                end
            else
                print('Invalid input to ' .. debug.getinfo(1, "n").name ..
                          ' : v.quantity, v.sealane, v.class or v.side are incorrect')
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return sealaneships
end
function CourseFromSeaLane(sealane)
    local startloc, course = nil, nil
    if sealane ~= nil then
        course = CourseOnSeaLane(sealane)
        --- print('CourseRandomOffSet(course, 25) disabled')
        course = CourseRandomOffSet(course, 25)
        local vector = 180
        if Count(course) > 1 then
            local bearing = Tool_Bearing({ latitude=course[1].latitude, longitude=course[1].longitude},
                {latitude=course[2].latitude, longitude=course[2].longitude })
            if bearing > 180 then vector = math.floor(360-bearing); else vector = math.floor(bearing + 180); end
        end
        startloc = PointOffset(course[1], 1, vector)
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return startloc, course
end
function CourseOnSeaLane(sealane)
    local pointC = nil
    local newwaypoints = {}
    local rnd_dist
    --chose direction
    local rnd_direction = math.random(1,2)
    if sealane ~= nil then
        rnd_dist = math.random(CourseLength (sealane))
        -- sealane:         1----2----3----4----5----6----7----8 total 800nm
        -- rnd_dist(450nm):                   ^
        --                                  pointC
        -- newcourse:                         1-2----3----4----5
        -- or               5----4----3----2--1
        for i=1, Count(sealane) - 1 do
            local leglength = Tool_Range(sealane[i], sealane[i+1])
            if rnd_dist > leglength then
                rnd_dist = rnd_dist - leglength
                if rnd_direction == 1 then table.insert(newwaypoints, sealane[i]) end
            elseif pointC == nil then
                local brng = Tool_Bearing(
                    { latitude=sealane[i].latitude, longitude=sealane[i].longitude},
                    { latitude=sealane[i+1].latitude, longitude=sealane[i+1].longitude })
                pointC = World_GetPointFromBearing(
                    {latitude = sealane[i].latitude, longitude = sealane[i].longitude
                    , distance = rnd_dist, bearing = brng})
                pointC['TypeOf'] = 'ManualPlottedCourseWaypoint'
                table.insert(newwaypoints, pointC)
                if rnd_direction == 1 then  break; end
            elseif rnd_direction == 2 then
                table.insert(newwaypoints, sealane[i])
            end
        end
        if rnd_direction == 2 then      -- add last point
            table.insert(newwaypoints, sealane[Count(sealane)])
        elseif rnd_direction == 1 then  -- invert table (https://gist.github.com/balaam/3122129)
            for i=1, math.floor(#newwaypoints / 2) do
                newwaypoints[i], newwaypoints[#newwaypoints - i + 1] = newwaypoints[#newwaypoints - i + 1], newwaypoints[i]
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return newwaypoints
end
function CourseLength(sealane)
    local distance = 0
    if sealane ~= nil then
        for i=1, Count(sealane) - 1 do
            distance = distance + Tool_Range(sealane[i], sealane[i+1])
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return distance
end
print('Loaded CommercialDeployment lua')