-- ========================================================================
-- Generation Functions
-- ========================================================================
function PrepareSealanesBetweenPorts(ports, playerside, ...)
    local args = {...}
    local ship_guids = {};
    if ports ~= nil and playerside ~= nil and args ~= nil then
        args = args[1]
        local pt, retval,shipT
        for k,v in pairs(args) do
            pt = FilterTable(ports, {{ name = v.source }, {name = v.destination}})
            if (Count(pt) == 2 ) then
                retval,shipT = pcall(ScenEdit_AddUnit,{type ='Ship', name ='Commercial Container Vessel ', dbid =2026, side =playerside, throttle='creep', Latitude=pt[1].harbor.latitude,Longitude=pt[1].harbor.longitude}) ;
                if not retval or shipT ~= nil then
                    table.insert(ship_guids, 
                        {sealane=v.source:gsub("%s+", "") .. "_to_".. v.destination:gsub("%s+", ""), 
                         guid = shipT.guid})
                    shipT = ScenEdit_GetUnit({name='Commercial Container Vessel ', guid=shipT.guid});
                    shipT.course = {[1] = { latitude = pt[2].harbor.latitude, TypeOf = 'ManualPlottedCourseWaypoint', longitude = pt[2].harbor.longitude }}
                end
            else
                print( "error - two valid ports are needed ")
                print( v )
                print( "found" )
                print( pt )
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
        if ports == nil then print('ports == nil'); end
        if args == nil then print('args == nil'); end
        if playerside == nil then print('playerside == nil'); end
    end

    return ship_guids
end
function TransferCompletedFindersToSealanes(finders, sealanes)
    local newsealanes = sealanes;
    local remainingfinders = finders
    if sealanes ~= nil and finders ~= nil then
        remainingfinders = {}
        for k,v in pairs(finders) do
            local u = ScenEdit_GetUnit({name='Commercial Container Vessel ', guid=v.guid});
            if u == nil then
                print("Error retrieving ".. k .. " with guid " .. v.guid)
            elseif Count(u.course) < 2 then
                table.insert(remainingfinders,v)
            else
                if InTablebyFilter(newsealanes, {{name =v.sealane}}) then
                    local idx = -1
                    for p, q in pairs(newsealanes) do
                        if q.name == v.sealane then
                            idx = p
                        end
                    end
                    if idx > 0 then table.remove(newsealanes, idx); print('replaced ' .. v.sealane ) end
                end
                table.insert(newsealanes, {name = v.sealane, course = u.course, length = CourseLength(u.course)})
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return remainingfinders, newsealanes
end
function GenerateSealanesFinder(sealanesbuilder, ports, ...)
    local args = {...}
    local unknownsealanes = {}
    local newfinders = {}
    local ExitMenu = false
    if ports ~= nil and sealanesbuilder ~= nil and args ~= nil then
        for k, v in pairs(args[1]) do
            local sealanename=v.source:gsub("%s+", "") .. "_to_".. v.destination:gsub("%s+", "")
            -- If not in sealanesbuilder.sealanefinders or sealanesbuilder.sealanes
            if sealanesbuilder.sealanefinders[sealanename] == nil and sealanesbuilder.sealanes[sealanename] == nil then
                table.insert(unknownsealanes, v)
            end
        end
        if Count(unknownsealanes) == 0 then
            ScenEdit_MsgBox('These sea lanes are completed or awaiting calculation.', 1)
        else
            local message = "The following sea lanes will begin calculation:\n"
            for k, v in pairs(unknownsealanes) do
                local sealanename=v.source:gsub("%s+", "") .. "_to_".. v.destination:gsub("%s+", "")
                message = message .. sealanename .. '\n'
            end
            message = message .. '\nPlease ensure the simulation is started and not paused.\n\n'
                              .. 'Switch to "' .. sealanesbuilder.playerside .. '" to monitor route calculations.\n'
                              .. '=====  Attention ======\n'
                              .. 'Calculations will not occur while the Sea Lane Builder Dialog is running; therefore,\n'
                              .. 'Sea Lane Build will exit.  You can restart to check progress, but sea lanes will not calculate while running.\n'
            if ScenEdit_MsgBox(message, 1) == 'OK' then
                local retval,side = pcall(ScenEdit_GetSideOptions,{side=sealanesbuilder.playerside}) ;
                print(side)
                if not retval or side == nil then
                    ScenEdit_AddSide({name=sealanesbuilder.playerside})
                    side = ScenEdit_GetSideOptions({side=sealanesbuilder.playerside})
                end
                newfinders = PrepareSealanesBetweenPorts(ports, sealanesbuilder.playerside, unknownsealanes)
            end
            ExitMenu = true
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return newfinders, ExitMenu
end
function ResetCalculations(sealanesbuilder)
    if sealanesbuilder.deployedships ~= nil then
        sealanesbuilder.deployedships = DeleteUnits(sealanesbuilder.deployedships)
    end
    if sealanesbuilder.deployedrefpnts ~= nil then
        sealanesbuilder.deployedrefpnts = DeleteRefPoints(sealanesbuilder.deployedrefpnts)
    end
    sealanesbuilder = { sealanefinders={}, sealanes={}, portsfilter={}, deployedships = {}, deployedrefpnts = {} , playerside = '' };

    return sealanesbuilder
end

print('Loaded SealaneGeneration lua')