-- ========================================================================
-- Sea Lane Generation Demo Functions and Menus
-- ========================================================================
function MenuPopulateSealane(ships, sealanes, shipclasses, playerside)
    local result = {}
    if ships ~= nil and sealanes ~= nil and playerside ~= nil then
        local sealanenm = 0
        for k, v in pairs(sealanes) do
            sealanenm = sealanenm +  CourseLength(v.course)
        end
        sealanenm =  Round(sealanenm)
        local democount = math.floor(sealanenm * 3 / 100)
        local message = 'Enter the number of units to deploy per 100nm of the ' .. sealanenm .. 
                        ' miles of sealanes for each of the following class: ' .. 
                        'Container ship, Cargo ship, and Bulk carrier.\n'
        message = message .. 'Be conservative, 1 per 100nm = '.. democount .. ' ships.'
        local merchantdensityper100nm = RequestNumber(message)
        if merchantdensityper100nm ~= nil then
            -- This uses the commercial ships to populate the sealanes at X number of ships per 100 miles to the side identified.
            local newships = PopulateSealanes(ships, sealanes, merchantdensityper100nm,
                                            shipclasses,
                                            playerside)
            result = newships
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end
function SelectSeaLaneMenu(sealanes, titlestring)
    local result = SelectObjectsMenu(sealanes, 10, titlestring, 'sea lane', 'name')

    return result
end
function SelectObjectsMenu(T,pagersize,titlestring, objectname, attribute)
    local result = {}
    if T ~= nil then
        local currentpage = 1
        local selectdone = false
        repeat
            local message = titlestring
            message = message .. '\n There ' .. IsAre(T) ..' ' .. Count(T) .. ' ' .. objectname ..':\n'
            message = message .. ' -----------------------------------------\n'
            local pagerrows = TablePager(T, pagersize, currentpage)
            local itemid = pagersize * (currentpage - 1)
            local keytable = {}
            for k, v in pairs(pagerrows) do
                itemid = itemid + 1
                keytable[itemid] = k
                if type(k) == 'number' and attribute == nil then
                    message = message .. itemid .. ') ' .. tostring(v) .. '\n'
                elseif type(k) == 'number' and attribute ~= nil then
                    message = message .. itemid .. ') ' .. v[attribute] .. '\n'
                else
                    message = message .. itemid .. ') ' .. k .. '\n'
                end
            end
            -- pad message 
            for x = Count(pagerrows), pagersize do
                message = message .. '\n'
            end
            message = message .. ' ----------- page '.. currentpage ..' of '.. math.ceil((Count(T)/pagersize)) ..' ---------------\n'
            message = message .. ' Select a ' .. objectname ..' by entering the item number or:\n'
            if Count(T) > pagersize then
                message = message .. '  a) <-- Previous Page        s) Next Page -->\n'
            end
            message = message .. ' -1) Select all\n'
            message = message .. '  0) Do nothing\n'
            local option = ScenEdit_InputBox (message)
            if option == 'a' and currentpage > 1 then
                currentpage = currentpage - 1
            elseif option == 's' and currentpage < math.ceil((Count(T)/pagersize)) then
                currentpage = currentpage + 1
            elseif option == '' then
                selectdone = true
            else
                local inputnum = tonumber(option)
                if inputnum ~= nil and type(inputnum) == 'number' then
                    if inputnum == -1 then
                        result = T
                        selectdone = true
                    elseif inputnum == 0 then
                        result = {}
                        selectdone = true
                    elseif inputnum >= (pagersize * (currentpage - 1) + 1) and inputnum <= (pagersize * currentpage) then
                        table.insert(result,T[keytable[inputnum]])
                        selectdone = true
                    else
                        selectdone = false
                    end
                end
            end
        until selectdone
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end
function EnterCustomSeaLaneMenu(sealanesbuilder, ports)
    local result = nil
    if sealanesbuilder ~= nil and ports ~= nil then
        local sealanebuilt = false
        local newsealanes = {}
        repeat
            local message = 'Welcome to the Sea Lane Builder\n'
                     .. '-----------------------------------------\n' 
                     .. 'Enter the desired source and destination ports '
                     .. 'of the ' .. Count(ports) .. ' ports in the global ports list.\n'
                     .. 'The format can accept:\n{source = "TBD", destination = "TBD"} or \n'
                     .. '{source = "TBD", destination = "TBD"},{source = "TBD", destination = "TBD"}\n'
                     .. '-----------------------------------------\n'
                     .. 'Current Selection: \n'
                    if Count(newsealanes) > 0 then
                        message = message .. TableToString(newsealanes) .. '\n'
                                          .. '-----------------------------------------\n'
                                          .. 'Add another'
                    else
                        message = message .. '-none-\n'
                                          .. '-----------------------------------------\n'
                                          .. 'Enter the'
                    end
                    message = message.. 'desired sea lane end points using the format above or\n'
                                     .. ' 0) Do nothing\n'
            if Count(newsealanes) > 0 then
                message = message .. ' 1) Confirm and begin plot(s)\n'
            end
            

            local option = ScenEdit_InputBox(message)
            if option == nil or option == '' or string.byte(option) == 48 then
                -- 0 selected
                sealanebuilt = true
            elseif string.byte(option) == 49 then
                -- 1 selected
                print('Proceed and plot')
                sealanebuilt = true
                result = newsealanes
            elseif option ~= nil and string.len(option) > 1 and string.find(option,',') > 1 then
                local userinput = StringToTable(option)
                local errormsg = nil
                for k, v in pairs(userinput) do
                    if v.source ~= nil and v.destination ~= nil then
                        local pt = FilterTable(ports, {{ name = v.source }, {name = v.destination}})
                        if (Count(pt) == 2 ) then
                            table.insert(newsealanes, v)
                        else
                            if errormsg == nil then errormsg = '' end
                            errormsg = errormsg .. 'error - two valid ports are needed '
                                                .. TableToString(v) .. ' is invalid'
                        end
                    end
                end
                if errormsg ~= nil then
                    print(errormsg)
                end
            end
        until sealanebuilt
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result

end
function EnterCustomDeploymentMenu(sealanesbuilder, ships)
    local result = nil
    if sealanesbuilder ~= nil and ships ~= nil then
        local deploymentbuilt = false
        local deployments = {}
        repeat
            local message = 'Welcome to the Sea Lane Builder\n'
                     .. '-----------------------------------------\n' 
                     .. 'Enter the desired sea lane name, ship type, quantity, and side'
                     .. 'The format can accept:\n{class=\'Container ship\', quantity= 10,'
                     .. ' sealane=\'Odesa_to_Marseille\', side=\'red\'}, or \n'
                     .. 'multiples separated by a comma {...},{...}\n'
                     .. '-----------------------------------------\n'
                     .. 'Current Selection: \n'
                    if Count(deployments) > 0 then
                        for k, v in pairs(deployments) do
                            message = message .. v.name .. ' ' .. v.quantity .. ' ' .. v.class .. '\n'
                        end
                        message = message .. '-----------------------------------------\n'
                                          .. 'Add another'
                    else
                        message = message .. '-none-\n'
                                          .. '-----------------------------------------\n'
                                          .. 'Enter the'
                    end
                    message = message.. 'desired deployment using the format above or\n'
                                     .. ' 0) Do nothing\n'
            if Count(deployments) > 0 then
                message = message .. ' 1) Confirm and begin deployment(s)\n'
            end
            local deploymentstring = ScenEdit_InputBox(message)
            if deploymentstring == nil or deploymentstring == '' or string.byte(deploymentstring) == 48 then
                -- 0 selected
                deploymentbuilt = true
            elseif string.byte(deploymentstring) == 49 then
                -- 1 selected
                print('Proceed and plot')
                deploymentbuilt = true
                result = deployments
            elseif deploymentstring ~= nil and string.len(deploymentstring) > 1 and string.find(deploymentstring,',') > 1 then
                local deploymenttable = StringToTable(deploymentstring)
                for k, v in pairs(deploymenttable) do
                    if InTablebyFilter(ships, {{class = v.class}}) and
                       InTablebyFilter(sealanesbuilder.sealanes, {{name = v.sealane}}) and
                       v.side ~= nil and (string.len(v.side) > 0) and v.quantity > 0 then
                            v.name = v.sealane
                            v.sealane = FilterTable(sealanesbuilder.sealanes, {{name = v.sealane}})[1].course
                            table.insert(deployments, v)
                    end
                end

            end
        until deploymentbuilt
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result

end
function GetMainMenuOption(sealanefinders, sealanes,currentdeployed, currentrefpnts, playerside)
    local result = nil
    if sealanefinders ~= nil  and sealanes ~= nil and currentdeployed ~= nil and currentrefpnts ~= nil and playerside ~= nil then
        local message = 'Welcome to the Sea Lane Builder\n'
                     .. '-----------------------------------------\n'
        if Count(sealanefinders) == 0 and Count(sealanes) == 0 then
            message = message .. 'Current Sea Lanes:  None\n'
                              .. '-----------------------------------------\n'
        else
            if Count(sealanefinders) > 0 then
                message = message .. 'Sea lane finders\n'
                                  .. ' There ' .. IsAre(sealanefinders) ..' ' .. Count(sealanefinders) .. ' sea lanes pending:\n'
                                  .. ' Is the simulation started and this tool exited to allow calculations?\n'
                                  .. '-----------------------------------------\n'
            end
            if Count(sealanes) > 0 then
                local totallength = 0
                for k, v in pairs(sealanes) do
                    totallength = totallength + Round(CourseLength(v.course), 1)
                end
                message = message .. ' There ' .. IsAre(sealanes) ..' ' .. Count(sealanes) .. ' sea lanes calculated:\n'
                                  .. ' ___________________ totaling ' .. totallength .. 'nm\n'
                                  .. '\n-----------------------------------------\n'
            end
        end
        if currentdeployed~= nil and Count(currentdeployed) >0 then
            message = message .. 'Current Sea Lane Traffic:\n'
            for k, v in pairs(GroupBy(currentdeployed, 'side')) do
                message = message .. '   ' .. k .. ' = ' .. v .. '\n'
            end
        else
            message = message .. 'Current Sea Lane Traffic:  None\n'
        end
        message = message .. '-----------------------------------------\n'
                          .. 'Sea Lanes\n'
                          .. '   1) List all sea lanes (includes lua print).\n'
                          .. '   2) Create demo in Mediterranean-Black Sea.\n'
                          .. '   3) Add custom sea lane.\n'
                          .. '   4) Delete sea lane(s) \n'
                          .. '-----------------------------------------\n'
                          .. 'Commercial Traffic\n'
                          .. '  14) Deploy traffic using a density number.\n'
                          .. '  15) Deploy traffic using the demo:\n        Container ship: 10 red, 10 blue,\n'
                          .. '            40 ' .. playerside ..'\n           Cargo ship: 10 red, 10 blue,\n'
                          .. '            40 ' .. playerside ..'\n'
                          .. '  16) Build custom deployment.\n'
                          .. '  17) Delete deployed traffic.\n'
                          .. '-----------------------------------------\n'
                          .. 'Reference Points: \n'
                          .. '  18) Plot sea lane reference points.\n'
                          .. '  19) Delete sea lane reference points.\n'
                          .. '------------------------------.\n'
                          .. '  33) Clear all data to restart.\n'
                          .. '------------------------------.\n'
                          .. '  0) Exit.\n'
        result = RequestNumber(message)
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end
function MainMenu( sealanesbuilder, ports, commships)
    local ExitMenu = false
    if sealanesbuilder ~= nil and ports ~= nil and commships ~= nil and 
      sealanesbuilder.sealanefinders ~= nil and sealanesbuilder.sealanes ~= nil 
      and sealanesbuilder.playerside ~= nil then
        local newfinders
        local sealanerequest = {{source='Casablanca', destination='Alexandroupoli'}, {source ='Gabes', destination = 'Limassol'},
                                {source ='Odesa', destination = 'Alexandroupoli'},{source ='Istanbul', destination = 'Odesa'},
                                {source ='Istanbul', destination = 'Casablanca'},{source ='Haifa', destination = 'Marseille'}}
        local shipclasses = {'Container ship', 'Cargo ship', 'Bulk carrier'}
        local odesa_to_marseille
        if #FilterTable(sealanesbuilder.sealanes, {{name = 'Odesa_to_Marseille'}}) > 0 then
            odesa_to_marseille = FilterTable(sealanesbuilder.sealanes, {{name = 'Odesa_to_Marseille'}})[1].course
        end
        local newshiprequests = {
            {class='Cargo ship', quantity= 10, sealane=odesa_to_marseille, side='blue'},
            {class='Container ship', quantity= 40, sealane=odesa_to_marseille, side=sealanesbuilder.playerside},
            {class='Bulk carrier', quantity= 40, sealane=odesa_to_marseille, side=sealanesbuilder.playerside}
            }
        local option = GetMainMenuOption(sealanesbuilder.sealanefinders, sealanesbuilder.sealanes,
                                sealanesbuilder.deployedships, sealanesbuilder.deployedrefpnts, sealanesbuilder.playerside)

        if option == nil or (option ~= nil and option == 0)  then
            ExitMenu = true
        elseif option == 1 then
            -- '1) List all sea lanes (includes lua print).\n'
            local sealanetoprint = SelectSeaLaneMenu(sealanesbuilder.sealanes, 'Print Sea Lane to Console')
            if Count(sealanetoprint) > 0 then
                print('=============================================================')
                print('=============================================================')
                print(TableToString(sealanetoprint))
            end
        elseif option == 2 then
            -- '2) Create demo sea lanes in Mediterranean\Black Sea.\n'
            newfinders, ExitMenu = GenerateSealanesFinder(sealanesbuilder,ports, sealanerequest)
            sealanesbuilder.sealanefinders = MergeTables(sealanesbuilder.sealanefinders, newfinders)
        elseif option == 3 then
            -- '3) Add custom sea lane.\n'
            sealanerequest = EnterCustomSeaLaneMenu(sealanesbuilder, ports)
            if sealanerequest ~= nil and Count(sealanerequest) > 0 then
                newfinders, ExitMenu = GenerateSealanesFinder(sealanesbuilder,ports, sealanerequest)
                sealanesbuilder.sealanefinders = MergeTables(sealanesbuilder.sealanefinders, newfinders)
            end
        elseif option == 4 then
            -- '4) Delete sea lane(s) \n'
            local sealanestodelete = SelectSeaLaneMenu(sealanesbuilder.sealanes, 'Select the sea lane(s) to delete')
            sealanesbuilder.sealanes = DeleteSeaLanes(sealanesbuilder.sealanes, sealanestodelete)
        elseif option == 14 then
            -- '14) Deploy traffic using a density number.\n'
            local newships = MenuPopulateSealane(commships, sealanesbuilder.sealanes, shipclasses, sealanesbuilder.playerside)
            sealanesbuilder.deployedships = MergeTables(sealanesbuilder.deployedships, newships)
        elseif option == 15 then
            -- '15) Deploy traffic using the demo: Container ship: 10 red 10 blue 40 playerside Cargo ship: 10 red 10 blue  40 playerside
            local newships = DeployTraffic(commships, newshiprequests)
            sealanesbuilder.deployedships = MergeTables(sealanesbuilder.deployedships, newships)
        elseif option == 16 then
            -- '16) Build custom deployment.\n'
            local newdeployments = EnterCustomDeploymentMenu(sealanesbuilder, commships)
            print('===========================================================')
            print('New deployments')
            print (newdeployments)
            local newships= DeployTraffic(commships, newdeployments)
            sealanesbuilder.deployedships = MergeTables(sealanesbuilder.deployedships, newships)
        elseif option == 17 then
            -- '17) Delete deployed traffic.\n'
            sealanesbuilder.deployedships=DeleteUnits(sealanesbuilder.deployedships)
        elseif option == 18 then
            -- '18) Plot sea lane reference points.\n'
            local sealanestoplot = SelectSeaLaneMenu(sealanesbuilder.sealanes,'Select sea lane to plot:\n')
            for k, v in pairs(sealanestoplot) do
                sealanesbuilder.deployedrefpnts = MergeTables(sealanesbuilder.deployedrefpnts, PlotSealaneWaypoints(v, sealanesbuilder.playerside))
            end
        elseif option == 19 then
            -- '19) Delete sea lane reference points.\n'
            sealanesbuilder.deployedrefpnts = DeleteRefPoints(sealanesbuilder.deployedrefpnts)
        elseif option == 33 then
            -- '33) Clear all data to restart.\n'
            sealanesbuilder = ResetCalculations(sealanesbuilder)
            sealanesbuilder.sealanes = {}
        end
    else
        local errormessage = 'Invalid input to ' .. debug.getinfo(1).name .. ' defined on line ' .. debug.getinfo(1).linedefined .. ' with params:'
        for i = 1,debug.getinfo(1).nparams do
            local name, value = debug.getlocal(1, i)
            if name ~= nil then
                errormessage = errormessage .. ' ' .. tostring(name) .. ' = ' .. tostring(value) .. ','
            end
        end
        ExitMenu = true
        print(errormessage)
    end

    return sealanesbuilder, ExitMenu
end
-- ========================================================================
-- Main()
-- ========================================================================



function SeaLaneMainMenu()
    if GLOBAL_ALLPORTS == nil or GLOBAL_COMMERCIALSHIPS == nil then
        ScenEdit_MsgBox('You need to ensure GLOBAL_ALLPORTS and ' 
                    .. 'GLOBAL_COMMERCIALSHIPS are loaded before running the script.'
                    .. 'This can be in the LoadScenario event or by placing the '
                    .. '/CommercialShipping/CommercialData.lua in the lua folder'
                    .. ' and loading using ScenEdit_RunScript("/CommercialShipping/CommercialData.lua")', 1)
    else
        if GLOBAL_SEALANEGEN == nil then
            GLOBAL_SEALANEGEN = { sealanefinders={}, sealanes={}, portsfilter={}, deployedships = {}, deployedrefpnts = {}, playerside = '' };
        end
        if GLOBAL_SEALANES ~= nil then
            GLOBAL_SEALANEGEN.sealanes = GLOBAL_SEALANES
        end
        local ExitBuildTool = false
        repeat
            GLOBAL_SEALANEGEN.playerside = 'Commercial Shipping'
            GLOBAL_SEALANEGEN.sealanefinders, GLOBAL_SEALANEGEN.sealanes = TransferCompletedFindersToSealanes(GLOBAL_SEALANEGEN.sealanefinders, GLOBAL_SEALANEGEN.sealanes )
            GLOBAL_SEALANEGEN, ExitBuildTool = MainMenu( GLOBAL_SEALANEGEN, GLOBAL_ALLPORTS, GLOBAL_COMMERCIALSHIPS)
        until ExitBuildTool
    end
end


print('Loaded Sealane Menus lua')