-- ========================================================================
-- Utility Functions
-- ========================================================================
-- Note: copied from LuaInit in game file
function Round(num, numDecimalPlaces)
    local result = nil
    if num ~= nil then
        local mult = 10^(numDecimalPlaces or 0)
        result =math.floor(num * mult + 0.5) / mult
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end

function Count(T)
    local count = 0
    if T == nil then
        count = nil
    elseif type(T) == 'table' then
        for _ in pairs(T) do count = count + 1 end
    else
        print(type(T))
        count = #T
    end

    return count
end

function Between(val, x, y)
    local result = false
    if val ~= nil then
      if x > y then
          result = (val <= x) and (val >= y)
      else
          result = (val <= y) and (val >= x)
      end
    end

    return result
end

function MatchFilter(object, filter)
    local result = false
    if filter ~= nil then
        for k,v in pairs(filter) do 
            if object[k] == v then
                result = true
            else
                return false
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end

function TablePager(T, pagesize, pageid)
    local result = {}
    if T ~= nil and pagesize ~= nil and type(pagesize) == 'number' and pageid ~= nil and type(pageid) == 'number' then
        local rowcount = Count(T)
        local pagecount = (rowcount / pagesize)
        local currrentrow = 0
        for k, v in pairs(T) do
            currrentrow = currrentrow + 1
            if currrentrow > (pagesize * (pageid - 1)) and currrentrow <= (pagesize * pageid) then
                result[k]=v
            elseif currrentrow > (pagesize * pageid) then
                break
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end

function TableFind(T,a)
    local result = -1
    if T ~= nil and a ~= nil then
        local index = 0
        for k, v in pairs(T) do
            index = index + 1
            if ((type(a) == "table") and (type(v) == "table") and
                TableToString(a) == TableToString(v)) or ((type(a) == type(v)) and a == v) then
                result = index
                break
            end
        end
    end

    return result
end

function MergeTables(a, b)
    if a == nil then a = {}; end
    if b == nil then b = {}; end
    local result = {}
    for k,v in pairs(a) do
        table.insert(result,v)
    end
    for k,v in pairs(b) do
        if TableFind(result,v) < 1 then
            table.insert(result,v)
        end
    end

    return result
end

function FilterTable( T , filters )
    -- function to filter the data using key and value pairs
    -- ex: FilterTable(ports, {{name='value', Facilities_RoRo='Yes'},{country='value', Cranes_Container='Yes' }})
    -- ex: FilterTable(ships, {{class = 'Trawler'}})
    local result = nil
    if T ~= nil then 
        local selectedrows = {};
        for k,v in pairs(T) do
            if filters == nil or Count(filters) == 0 then
                table.insert(selectedrows,v)
            else
                for i=1,Count(filters) do
                    if MatchFilter(v, filters[i]) then
                        table.insert(selectedrows,v)
                        break
                    end
                end
            end
        end
        result = selectedrows
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end

function InTable(T, x)
    local result = false
    if T ~= nil then
        for k, v in pairs(T) do
            if type(v) == 'table' and InTable(v,x) then
                result = true
                break
            elseif v == x then
                result = true
                break
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return false
end

function InTablebyFilter(T, filter)
    local result = false
    local sh = FilterTable(T, filter)
    if Count(sh) > 0 then
        result = true
    end

    return result
end
function IsAre(T)
    local result = 'is'
    if T ~= nil and Count(T) > 1 then
        result = 'are'
    end

    return result
end
function GroupBy(T, key)
    -- counts the number of unique values for the input key
    -- ex GroupBy(ships, 'flag')
    local valuecounts = {};
    local valuename = ''
    if T ~= nil then
        for k,v in pairs(T) do
            valuename = v[key]
            if valuecounts[valuename] ~= nil then
                valuecounts[valuename] = valuecounts[valuename] + 1
            else
                valuecounts[valuename] = 1
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return valuecounts
end

function RequestNumber(message)
    local result, strnum, inputnum = nil, nil, nil
    if message ~= nil then
        repeat
            strnum = ScenEdit_InputBox (message)
            inputnum = tonumber(strnum)
            if strnum == '' then
                return result
            elseif inputnum ~= nil and type(inputnum) == 'number' and inputnum > -1 then
                result = inputnum
            end
        until result ~= nil
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end

function TableToString(T)
    -- https://gist.github.com/justnom/9816256
    local result = "{"
    if T ~= nil then
        for k, v in pairs(T) do
            -- Check the key type (ignore any numerical keys - assume its an array)
            if type(k) == "string" then
                result = result.. k .. "="
            end

            -- Check the value type
            if type(v) == "table" then
                result = result..TableToString(v)
            elseif type(v) == "boolean" then
                result = result..tostring(v)
            elseif type(v) == "number" then
                result = result.. v
            else
                result = result.."\""..v.."\""
            end
            result = result..","
        end
        -- Remove leading commas from the result
        if result ~= "" then
            result = result:sub(1, result:len()-1)
        end
    end

    return result.."}"
end

function StringToTable(input)
    local filter={}
    for w in string.gmatch(input, "%b{}") do
        local t = {} --gsub("%s+", "")
        for k, v in string.gmatch(w, "'*([_%/.%w]+)'*%s*=%s*'*([_%w/.%s]+)'*") do
            k = k:gsub("^ ", ""):gsub(" $", "")
            v = v:gsub("^'", ""):gsub("'$", ""):gsub("^ ", ""):gsub(" $", "")
            local num = tonumber(v)
            if num ~= nil and type(num) == 'number'and num ~= "" then
                v = num
            end
            t[k] = v
        end
        table.insert(filter,t)
    end

    return filter
end
-- ========================================================================
-- Commercial Traffic Utility Functions
-- ========================================================================
function AddInitialHeading(guid)
	local unit = ScenEdit_GetUnit({guid=guid})
	local initialHeading = Tool_Bearing(
		{
			latitude=unit.latitude,
			longitude=unit.longitude
		},
		{
			latitude=unit.course[1].latitude,
			longitude=unit.course[1].longitude,
		}
	)
	ScenEdit_SetUnit({guid=guid,heading=initialHeading})

end

function OverWater (latitude, longitude)
    local result = false
	local pointElevation = World_GetElevation({
		latitude = latitude,
		longitude = longitude})
	if pointElevation < 0 then
		result = true
	end

    return result
end

function CourseRandomOffSet(course, max_radius )
    local distance = math.random(0.1,max_radius)
    local vector = math.random(1,360)
    local newcourse = {}
    for i=1,Count(course)-1 do
        local nextsegmentlength = Tool_Range(course[i], course[i+1])
        if nextsegmentlength/2 < distance then
            table.insert(newcourse, PointOffset(course[i], nextsegmentlength/2, vector))
        else
            table.insert(newcourse, PointOffset(course[i], distance, vector))
        end
    end
    table.insert(newcourse, course[Count(course)])

    return newcourse
end
function PointOffset(point, maxdistance, vector)
    local result = point
    if point ~= nil and vector ~= nil and maxdistance ~= nil and
        OverWater(point.latitude, point.longitude) then
        local testdistance = maxdistance
        local decrement = maxdistance *.01
        local newPoint = World_GetPointFromBearing(
        {latitude = point.latitude, longitude = point.longitude
        , distance = testdistance, bearing = vector})
        -- decrement test distance
        while not OverWater (newPoint.latitude, newPoint.longitude) and testdistance > 0 do
            testdistance = testdistance - decrement
            newPoint = World_GetPointFromBearing(
                {latitude = point.latitude, longitude = point.longitude
                , distance = testdistance, bearing = vector})
        end
        if testdistance < maxdistance then
            testdistance = testdistance / 2
        end
        result = World_GetPointFromBearing(
                {latitude = point.latitude, longitude = point.longitude
                , distance = testdistance, bearing = vector})
    end
    return result
end
function FindWater(point)
    local result = nil
    if point ~= nil then
      for x = 1, 100 do
        local r = x * .1
        local randomisationCircle = World_GetCircleFromPoint({
            latitude=point.latitude,
             longitude=point.longitude,
             radius=r,
             numpoints = 360})
        for i = 1, 360 do
            if OverWater(randomisationCircle[i].latitude, randomisationCircle[i].longitude) then
                result = {latitude = randomisationCircle[i].latitude,longitude = randomisationCircle[i].longitude}
                break
            end
        end
      end
    end

    return result
end

function FindTerrain(point, minelevation, maxelevation)
    local result = nil
    if point ~= nil and minelevation ~= nil and maxelevation ~= nil then
        for x = 1, 10000 do
            local r = x *.0001
            for i = 1, 360 do
                local angle = i * math.pi / 180
                local ptlongitude, ptlatitude = point.longitude + r * math.cos( angle ), point.latitude + r * math.sin( angle )
                local loc=World_GetLocation ({latitude = ptlatitude,longitude = ptlongitude})
                if loc.altitude ~= nil and Between(loc.altitude, minelevation, maxelevation) then
                    result = {latitude = ptlatitude,longitude = ptlongitude}
                    break
                end
            end
        end 
    end

    return result
end 
function FilterPortsInArea(ports, playerside, ...)
    -- Filters the ports value to those within the 
    -- the area defined as args[1] or if nil returns all 
    -- ex. portsel = FilterPortsInArea(ports, 'blue', {"RP-28560","RP-28561","RP-28562"})
    local args = {...}
    args = args[1]
    local selectedports = {};
    if ports ~= nil and playerside ~= nil then 
        for k,v in pairs(ports) do 
            if Count(args) == 0 then
            table.insert(selectedports,v)
            else
                --plot a unit
                local shipT = ScenEdit_AddUnit({type ='Ship', unitname =v.name .. " Buoy", dbid=347, side = playerside, Lat=v.harbor.latitude,Lon=v.harbor.longitude})
                shipT = ScenEdit_GetUnit({name=v.name .. " Buoy", guid=shipT.guid});
                if shipT ~= nil and shipT:inArea(args) then
                    table.insert(selectedports,v)
                end
                shipT:delete()
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return selectedports
end

function PrintPortList(selectedports)
    local portnames={}
    if selectedports ~= nil then
        for i=1, Count(selectedports) do
            table.insert(portnames, selectedports[i])
        end
        print(table.sort(portnames))
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

end
function PlotPorts(selectedports, playerside)
    if selectedports ~= nil and playerside ~= nil then
        for i=1, Count(selectedports) do
            -- ScenEdit_AddUnit({type ='Ship', unitname =selectedports[i].name .. " Buoy", dbid=347, side =playerside, Lat=selectedports[i].harbor.latitude,Lon=selectedports[i].harbor.longitude})
            local retval,shipT = pcall(ScenEdit_AddUnit,{type ='Ship', unitname =selectedports[i].name .. " Buoy", dbid=347, side =playerside, Lat=selectedports[i].harbor.latitude,Lon=selectedports[i].harbor.longitude}) ;
            if not retval or shipT ~= nil then

            end
            local retval,shipT = pcall(ScenEdit_AddUnit,{type ='Facility', unitname =selectedports[i].name , dbid=2980, side =playerside, Lat=selectedports[i].shore.latitude,Lon=selectedports[i].shore.longitude}) ;
            if not retval or shipT ~= nil then

            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

end
function PlotSealaneWaypoints(sealane, playerside)
    local refpnts = {}
    if sealane ~= nil and sealane.course ~= nil then
        for i=1, Count(sealane.course) do
            local rp = ScenEdit_AddReferencePoint( {side=playerside, name=sealane.name .. '_' .. i,
              lat=sealane.course[i].latitude, lon=sealane.course[i].longitude, highlighted=true})
            table.insert(refpnts, rp)
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return refpnts
end

function DeleteUnits(unitstable)
    local result = nil
    if unitstable ~= nil then
        for k, v in pairs(unitstable) do
            local status_success,retval = pcall(ScenEdit_DeleteUnit,{guid=v.guid}) ;
        end
        result = {}
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end
function DeleteSeaLanes(sealanes, sealanestodelete)
    local result = sealanes
    if sealanes ~= nil and sealanestodelete ~= nil 
    and Count(sealanestodelete) > 0 and Count(sealanes) > 0 then
        for k, v in pairs(sealanestodelete) do
            local removeid = nil
            print(v.name)
            for i, r in pairs(result) do
                if v.name == r.name then
                    removeid = i
                end
            end
            if removeid ~= nil then
                table.remove(result, removeid)
            end
        end
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end
function DeleteRefPoints(refpoints)
    local result = nil
    if refpoints ~= nil then
        for k, v in pairs(refpoints) do
            local status_success, retval = pcall(ScenEdit_DeleteReferencePoint,{side=v.side, name=v.name, guid=v.guid}) ;
        end
        result = {}
    else
        print('Invalid input to ' .. debug.getinfo(1, "n").name)
    end

    return result
end
print('Loaded Utility lua')